import re
from ast import literal_eval

from test_helper import run_common_tests, get_answer_placeholders, get_file_output, failed, passed


def maybe_eval(string):
    try:
        return literal_eval(string)
    except SyntaxError:
        return string

if __name__ == '__main__':
    run_common_tests()
    placeholders = get_answer_placeholders()
    output = get_file_output()
    oracle = '''expected {3, 4, 5}; actual {3, 4, 5}
expected {3, 4, 5}; actual {3, 4, 5}
expected {4, 5}; actual {4, 5}
expected {4, 5}; actual {4, 5}'''.split('\n')
    messages = '''The first set did not match the expected output.
The second set did not match the expected output.
The third set did not match the expected output.
The fourth set did not match the expected output.'''.split('\n')
    if len(output) != len(oracle):
        failed('Expected {expected} line(s) of program output, but the program printed {actual} line(s)'
               .format(expected=len(oracle), actual=len(output)))
    else:
        for actual, expected, message in zip(output, oracle, messages):
            actual_unordereds = tuple(maybe_eval(unordered) for unordered in re.findall('\{.*?\}', actual))
            expected_unordereds = tuple(maybe_eval(unordered) for unordered in re.findall('\{.*?\}', expected))
            if actual_unordereds != expected_unordereds:
                failed(message.format(actual=actual))
            elif re.sub('\{.*?\}', '�', actual) != re.sub('\{.*?\}', '�', expected):
                failed(message.format(actual=actual))
        else:
            passed()
